@@ -346,7 +346,7 @@ not supported."
          (sb-impl::finalizer-thread-stop)
          ;; Acquiring sb-thread::*make-thread-lock* prevents creation
          ;; of new threads.
-         (sb-thread::with-system-mutex (sb-thread::*make-thread-lock*)
+         (sb-int:with-system-mutex (sb-thread::*make-thread-lock*)
            ;; Dead threads aren't pruned from *ALL-THREADS* until the Pthread join.
            ;; Do that now so that the forked process has only the main thread
            ;; in *ALL-THREADS* and nothing in *JOINABLE-THREADS*.
@@ -510,11 +510,9 @@ not supported."
 (define-call "mlockall" int minusp (flags int))
 (define-call "munlockall" int minusp)
 
-#-win32
-(define-call "getpagesize" int minusp)
-#+win32
-;;; KLUDGE: This could be taken from GetSystemInfo
-(export (defun getpagesize () 4096))
+(export 'getpagesize)
+(declaim (inline getpagesize))
+(defun getpagesize () (extern-alien "os_reported_page_size" (unsigned 32)))
 
 ;;; passwd database
 ;; The docstrings are copied from the descriptions in SUSv3,
@@ -807,7 +805,8 @@ not supported."
              (if (minusp value)
                  (syscall-error 'utimes)
                  value)))
-      (let ((fun (extern-alien "sb_utimes" (function int (c-string :not-null t)
+      (let ((fun (extern-alien #-netbsd "utimes" #+netbsd "sb_utimes"
+                               (function int (c-string :not-null t)
                                                   (* (array alien-timeval 2)))))
             (name (filename filename)))
         (if (not (and access-time modification-time))
