@@ -11,7 +11,6 @@
 
 (in-package "SB-VM")
 
-#-sb-fluid
 (declaim (inline adjustable-array-p
                  array-displacement))
 
@@ -1093,6 +1092,8 @@ of specialized arrays is supported."
 
 ;;;; fill pointer frobbing stuff
 
+(setf (info :function :predicate-truth-constraint 'array-has-fill-pointer-p)
+      '(and vector (not simple-array)))
 (declaim (inline array-has-fill-pointer-p))
 (defun array-has-fill-pointer-p (array)
   "Return T if the given ARRAY has a fill pointer, or NIL otherwise."
@@ -1190,7 +1191,7 @@ of specialized arrays is supported."
   (let* ((old-length (length vector))
          (min-extension (or min-extension
                             (min old-length
-                                 (- sb-xc:array-dimension-limit old-length))))
+                                 (- array-dimension-limit old-length))))
          (new-length (the index (+ old-length
                                    (max 1 min-extension))))
          (fill-pointer (1+ old-length)))
@@ -1808,7 +1809,7 @@ function to be removed without further warning."
 
        ,@(ecase style
     (:call
-     `((!define-load-time-global ,table-name ,(make-array (1+ widetag-mask)))
+     `((!define-load-time-global ,table-name ,(sb-xc:make-array (1+ widetag-mask)))
 
        ;; This SUBSTITUTE call happens ** after ** all the SETFs below it.
        ;; DEFGLOBAL's initial value is dumped by genesis as a vector filled
