@@ -9,24 +9,20 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!SPARC-ASM")
+(in-package "SB-SPARC-ASM")
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(reg-tn-encoding) 'sb!vm)
+  (import '(reg-tn-encoding) "SB-VM")
   ;; Imports from SB-VM into this package
   (import '(;; SBs and SCs
-            sb!vm::zero sb!vm::immediate-constant
-            sb!vm::registers sb!vm::float-registers
-            sb!vm::control-registers
-            sb!vm::single-reg sb!vm::double-reg
+            sb-vm::zero sb-vm::immediate-constant
+            sb-vm::registers sb-vm::float-registers
+            sb-vm::control-registers
+            sb-vm::single-reg sb-vm::double-reg
             ;; TNs and offsets
-            sb!vm::zero-tn
-            sb!vm::zero-offset sb!vm::null-offset sb!vm::alloc-offset)))
-
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (setf *assem-scheduler-p* t)
-  (setf *assem-max-locations* 100))
+            sb-vm::zero-tn
+            sb-vm::zero-offset sb-vm::null-offset sb-vm::alloc-offset)))
 
 ;;; Constants, types, conversion functions, some disassembler stuff.
 (defun reg-tn-encoding (tn)
@@ -75,7 +71,7 @@ Otherwise, use the Sparc register names")
            (let ((offset (tn-offset loc)))
              (aver (zerop (mod offset 2)))
              (values (+ offset 32) 2)))
-          #!+long-float
+          #+long-float
           (long-reg
            (let ((offset (tn-offset loc)))
              (aver (zerop (mod offset 4)))
@@ -97,7 +93,7 @@ Otherwise, use the Sparc register names")
        (lambda (name)
            (cond ((null name) nil)
                  (t (make-symbol (concatenate 'string "%" name)))))
-       sb!vm::*register-names*)
+       sb-vm::*register-names*)
   "The Lisp names for the Sparc integer registers")
 
 (defparameter sparc-reg-symbols
@@ -128,12 +124,15 @@ about function addresses and register values.")
 
 (define-arg-type reg
   :printer (lambda (value stream dstate)
-               (declare (stream stream) (fixnum value))
+               (declare (fixnum value))
                (let ((regname (get-reg-name value)))
-                 (princ regname stream)
-                 (maybe-note-associated-storage-ref
-                  value 'registers regname dstate)
-                 (maybe-add-notes value dstate))))
+                 (cond (stream
+                        (princ regname stream)
+                        (maybe-note-associated-storage-ref
+                         value 'registers regname dstate)
+                        (maybe-add-notes value dstate))
+                       (t
+                        (operand regname dstate))))))
 
 (defparameter float-reg-symbols
   #.(coerce
@@ -142,23 +141,29 @@ about function addresses and register values.")
 
 (define-arg-type fp-reg
   :printer (lambda (value stream dstate)
-               (declare (stream stream) (fixnum value))
+               (declare (fixnum value))
                (let ((regname (aref float-reg-symbols value)))
-                 (princ regname stream)
-                 (maybe-note-associated-storage-ref
-                  value 'float-registers regname dstate))))
+                 (cond (stream
+                        (princ regname stream)
+                        (maybe-note-associated-storage-ref
+                         value 'float-registers regname dstate))
+                       (t
+                        (operand regname dstate))))))
 
 ;;; The extended 6 bit floating point register encoding for the double
 ;;; and long instructions of the sparc v9.
 (define-arg-type fp-ext-reg
   :printer (lambda (value stream dstate)
-               (declare (stream stream) (fixnum value))
+               (declare (fixnum value))
                (let* (;; Decode the register number.
                       (value (if (oddp value) (+ value 31) value))
                       (regname (aref float-reg-symbols value)))
-                 (princ regname stream)
-                 (maybe-note-associated-storage-ref
-                  value 'float-registers regname dstate))))
+                 (cond (stream
+                        (princ regname stream)
+                        (maybe-note-associated-storage-ref
+                         value 'float-registers regname dstate))
+                       (t
+                        (operand regname dstate))))))
 
 (define-arg-type relative-label
   :sign-extend t
@@ -256,9 +261,11 @@ about function addresses and register values.")
 
 (define-arg-type integer-condition-register
     :printer (lambda (value stream dstate)
-                 (declare (stream stream) (fixnum value) (ignore dstate))
+                 (declare (fixnum value))
                  (let ((regname (aref integer-condition-reg-symbols value)))
-                   (princ regname stream))))
+                   (if stream
+                       (princ regname stream)
+                       (operand regname dstate)))))
 
 (defconstant-eqx branch-predictions
   '(:pn :pt)
@@ -314,15 +321,19 @@ about function addresses and register values.")
 
 (define-arg-type fp-condition-register
     :printer (lambda (value stream dstate)
-                 (declare (stream stream) (fixnum value) (ignore dstate))
+                 (declare (fixnum value))
                  (let ((regname (aref fp-condition-reg-symbols value)))
-                   (princ regname stream))))
+                   (if stream
+                       (princ regname stream)
+                       (operand regname dstate)))))
 
 (define-arg-type fp-condition-register-shifted
     :printer (lambda (value stream dstate)
-                 (declare (stream stream) (fixnum value) (ignore dstate))
+                 (declare (fixnum value))
                  (let ((regname (aref fp-condition-reg-symbols (ash value -1))))
-                   (princ regname stream))))
+                   (if stream
+                       (princ regname stream)
+                       (operand regname dstate)))))
 
 (defun fp-condition (condition-reg)
   (or (position condition-reg fp-condition-registers)
@@ -359,6 +370,8 @@ about function addresses and register values.")
   (data   :field (byte 22 0) :reader format-2-unimp-data))
 
 (defconstant-eqx f3-printer
+  ;; FIXME: args are inverted if eliding display of source1,
+  ;; and I think totally backwards in any case.
   '(:name :tab
           (:unless (:same-as rd) rs1 ", ")
           (:choose rs2 immed) ", "
@@ -428,8 +441,8 @@ about function addresses and register values.")
 ;;
 (define-instruction-format
     (format-fpop2 32
-                  :default-printer #!-sparc-v9 '(:name :tab rs1 ", " rs2)
-                                   #!+sparc-v9 '(:name :tab rd ", " rs1 ", " rs2))
+                  :default-printer #-sparc-v9 '(:name :tab rs1 ", " rs2)
+                                   #+sparc-v9 '(:name :tab rd ", " rs1 ", " rs2))
   (op   :field (byte 2 30))
   (rd   :field (byte 5 25) :value 0)
   (op3  :field (byte 6 19))
@@ -488,9 +501,11 @@ about function addresses and register values.")
 
 (define-arg-type cond-move-condition-register
     :printer (lambda (value stream dstate)
-                 (declare (stream stream) (fixnum value) (ignore dstate))
+                 (declare (fixnum value))
                  (let ((regname (aref cond-move-condition-reg-symbols value)))
-                   (princ regname stream))))
+                   (if stream
+                       (princ regname stream)
+                       (operand regname dstate)))))
 
 ;; From the given condition register, figure out what the cc2, cc1,
 ;; and cc0 bits should be.  Return cc2 and cc1/cc0 concatenated.
@@ -563,9 +578,11 @@ about function addresses and register values.")
 
 (define-arg-type register-condition
     :printer (lambda (value stream dstate)
-                 (declare (stream stream) (fixnum value) (ignore dstate))
+                 (declare (fixnum value))
                  (let ((regname (aref cond-move-integer-condition-vec value)))
-                   (princ regname stream))))
+                   (if stream
+                       (princ regname stream)
+                       (operand regname dstate)))))
 
 (defconstant-eqx cond-move-integer-printer
   `(:name rcond :tab rs1 ", " (:choose immed rs2) ", " rd)
@@ -621,9 +638,6 @@ about function addresses and register values.")
 (define-bitfield-emitter emit-word 32
   (byte 32 0))
 
-(define-bitfield-emitter emit-short 16
-  (byte 16 0))
-
 (define-bitfield-emitter emit-format-1 32
   (byte 2 30) (byte 30 0))
 
@@ -842,7 +856,7 @@ about function addresses and register values.")
   ;; This instruction is called lduw for V9 , but looks exactly like ld
   ;; on previous architectures.
   (define-f3-inst ld #b11 #b000000 :load-store :load
-                  #!+sparc-v9 :print-name #!+sparc-v9 'lduw)
+                  #+sparc-v9 :print-name #+sparc-v9 'lduw)
 
   (define-f3-inst ldsw #b11 #b001000 :load-store :load) ; v9
 
@@ -946,7 +960,7 @@ about function addresses and register values.")
   (:emitter (emit-format-3-immed segment #b11 0 #b100001
                                  (reg-tn-encoding src1) 1 src2)))
 
-#!+sparc-64
+#+sparc-64
 (define-instruction ldxfsr (segment src1 src2)
   (:declare (type tn src1) (type (signed-byte 13) src2))
   (:printer format-3-immed ((op #b11) (op3 #b100001) (rd 1))
@@ -966,7 +980,7 @@ about function addresses and register values.")
   (:emitter (emit-format-3-immed segment #b11 0 #b100101
                                  (reg-tn-encoding src1) 1 src2)))
 
-#!+sparc-64
+#+sparc-64
 (define-instruction stxfsr (segment src1 src2)
   (:declare (type tn src1) (type (signed-byte 13) src2))
   (:printer format-3-immed ((op #b11) (op3 #b100101) (rd 1))
@@ -1030,7 +1044,7 @@ about function addresses and register values.")
 (define-instruction unimp (segment data)
   (:declare (type (unsigned-byte 22) data))
   (:printer format-2-unimp () :default :control #'unimp-control
-            :print-name #!-sparc-v9 'unimp #!+sparc-v9 'illtrap)
+            :print-name #-sparc-v9 'unimp #+sparc-v9 'illtrap)
   (:delay 0)
   (:emitter (emit-format-2-unimp segment 0 0 0 data)))
 
@@ -1170,7 +1184,7 @@ about function addresses and register values.")
   (:declare (type branch-condition condition)
             ;; KLUDGE: see comments in vm.lisp regarding
             ;; pseudo-atomic-trap.
-            #!-linux
+            #-linux
             (type (integer 16 31) target))
   (:printer format-3-immed ((op #b10)
                             (rd nil :type 'branch-condition)
@@ -1201,10 +1215,10 @@ about function addresses and register values.")
 ;;; breaking the disassembler if these are left in. The printer isn't
 ;;; terribly smart, but the emitted code is right. - CSR, 2002-08-04
 #+nil
-(define-instruction tcc (segment condition target &optional (cc #!-sparc-64 :icc #!+sparc-64 :xcc))
+(define-instruction tcc (segment condition target &optional (cc #-sparc-64 :icc #+sparc-64 :xcc))
   (:declare (type branch-condition condition)
             ;; KLUDGE: see above.
-            #!-linux
+            #-linux
             (type (integer 16 31) target)
             (type integer-condition-register cc))
   (:printer format-4-trap ((op #b10)
@@ -1375,9 +1389,9 @@ about function addresses and register values.")
                   (opf3 ,opf)
                   (rs1 nil :type ',(if extended 'fp-ext-reg 'fp-reg))
                   (rs2 nil :type ',(if extended 'fp-ext-reg 'fp-reg))
-                  #!-sparc-v9
+                  #-sparc-v9
                   (rd 0)
-                  #!+sparc-v9
+                  #+sparc-v9
                   (rd nil :type 'fp-condition-register))
         )
      (:dependencies
@@ -1607,13 +1621,6 @@ about function addresses and register values.")
      (integer
       (emit-word segment word)))))
 
-(define-instruction short (segment short)
-  (:declare (type (or (unsigned-byte 16) (signed-byte 16)) short))
-  :pinned
-  (:delay 0)
-  (:emitter
-   (emit-short segment short)))
-
 (define-instruction byte (segment byte)
   (:declare (type (or (unsigned-byte 8) (signed-byte 8)) byte))
   :pinned
@@ -1621,9 +1628,6 @@ about function addresses and register values.")
   (:emitter
    (emit-byte segment byte)))
 
-(define-bitfield-emitter emit-header-object 32
-  (byte 24 8) (byte 8 0))
-
 (defun emit-header-data (segment type)
   (emit-back-patch
    segment 4
@@ -1652,7 +1656,8 @@ about function addresses and register values.")
   (emit-chooser
    ;; We emit either 12 or 4 bytes, so we maintain 8 byte alignments.
    segment 12 3
-   (lambda (segment posn delta-if-after)
+   (lambda (segment chooser posn delta-if-after)
+       (declare (ignore chooser))
        (let ((delta (funcall calc label posn delta-if-after)))
          (when (<= (- (ash 1 12)) delta (1- (ash 1 12)))
            (emit-back-patch segment 4
