@@ -63,10 +63,7 @@ extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
 // masking out subtype_VectorWeakVisited.
 #define vector_subtype(header) (HeaderValue(header) & 7)
 // Test for presence of a bit in vector's header.
-// As a special case, if 'val' is 0, then test for all bits clear.
-#define is_vector_subtype(header, val) \
-  (subtype_##val ? (HeaderValue(header) & subtype_##val) : \
-   !(HeaderValue(header) & 7))
+#define is_vector_subtype(header, val) (HeaderValue(header) & subtype_##val)
 
 // Mask out the fullcgc mark bit when asserting header validity
 #define UNSET_WEAK_VECTOR_VISITED(v) \
@@ -83,12 +80,23 @@ extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
 /* Note: MAP-ALLOCATED-OBJECTS expects this value to be 1 */
 #define BOXED_PAGE_FLAG       1
 #define UNBOXED_PAGE_FLAG     2
+/* CONS_PAGE_FLAG doesn't get stored in the page table, though I am considering
+ * doing that. If conses went on segregated pages, then testing for a valid
+ * conservative root on a cons page is as simple as seeing whether the address
+ * is correctly aligned and lowtagged.
+ * Also, we could reserve bytes at the end of each page to act as a mark bitmap
+ * which is useful since conses are headerless objects, and one GC strategy
+ * demands mark bitmaps which are currently placed in a side table.
+ * That would unfortunately complicate the task of allocating a huge list,
+ * because hitting the line of demarcation between conses and the mark bits would
+ * require chaining the final cons to another page of conses and so on. */
+#define CONS_PAGE_FLAG        4
 #define OPEN_REGION_PAGE_FLAG 8
 #define CODE_PAGE_TYPE        (BOXED_PAGE_FLAG|UNBOXED_PAGE_FLAG)
 
 extern sword_t (*sizetab[256])(lispobj *where);
 #define OBJECT_SIZE(header,where) \
-  (is_cons_half(header)?2:sizetab[header_widetag(header)](where))
+  (is_header(header)?sizetab[header_widetag(header)](where):CONS_SIZE)
 
 lispobj *gc_search_space3(void *pointer, lispobj *start, void *limit);
 static inline lispobj *gc_search_space(lispobj *start, void *pointer) {
@@ -122,36 +130,6 @@ instance_scan(void (*proc)(lispobj*, sword_t, uword_t),
               lispobj bitmap, uword_t arg);
 
 extern int simple_fun_index(struct code*, struct simple_fun*);
-extern lispobj simple_fun_name(struct simple_fun*);
-
-#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-static inline lispobj funinstance_layout(lispobj* funinstance_ptr) { // native ptr
-    return instance_layout(funinstance_ptr);
-}
-static inline lispobj function_layout(lispobj* fun_ptr) { // native ptr
-    return instance_layout(fun_ptr);
-}
-static inline void set_function_layout(lispobj* fun_ptr, lispobj layout) {
-    instance_layout(fun_ptr) = layout;
-}
-#else
-static inline lispobj funinstance_layout(lispobj* instance_ptr) { // native ptr
-    // first 4 words are: header, trampoline, fin-fun, layout
-    return instance_ptr[3];
-}
-// No layout in simple-fun or closure, because there are no free bits
-static inline lispobj
-function_layout(lispobj __attribute__((unused)) *fun_ptr) { // native ptr
-    return 0;
-}
-static inline void set_function_layout(lispobj __attribute__((unused)) *fun_ptr,
-                                       lispobj __attribute__((unused)) layout) {
-    lose("Can't assign layout");
-}
-#endif
-
-#include "genesis/bignum.h"
-extern boolean positive_bignum_logbitp(int,struct bignum*);
 
 extern lispobj fdefn_callee_lispobj(struct fdefn *fdefn);
 
